/*
 *  EyeIndicator extension for GNOME shell
 *
 *  An eye following cursor for the GNOME shell panel.
 *
 *  Copyright 2012 canalguada <dguadal@free.fr>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
const Main = imports.ui.main;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Panel = imports.ui.panel;
const St = imports.gi.St;
const Clutter = imports.gi.Clutter;
const Cairo = imports.cairo;
 
const PanelMenu = imports.ui.panelMenu;

const UPDATE_INTERVAL = 100;
const EYE_WIDTH = 1.5;
const IRIS_WIDTH = 1.5;
const PUPIL_WIDTH = 0.0;
 
const MARGIN = 1;
 
const EyePosition = {
    CENTER: 0,
    RIGHT: 1,
    LEFT: 2
}

let eyeIndicator;

function main() {
	init();
	enable();
}

function init() {
}

function enable() {
	eyeIndicator = new EyeIndicator();
	eyeIndicator.setActive(true);

    Main.panel.addToStatusArea('eyeIndicator', eyeIndicator);
}

function disable() {
	eyeIndicator.setActive(false);
	eyeIndicator.destroy();
}

function EyeIndicator() {
	this._init();
}

EyeIndicator.prototype = {
	__proto__: PanelMenu.Button.prototype,

	_init: function() {
		PanelMenu.Button.prototype._init.call(this, St.Align.START);

	    this.area = new St.DrawingArea();
	 
	    this.area.set_width(Panel.PANEL_ICON_SIZE - 2 * MARGIN);
	    this.area.set_height(Panel.PANEL_ICON_SIZE - 2 * MARGIN);
	     
	    this.actor.set_width(Panel.PANEL_ICON_SIZE + 8);
	    this.actor.add_actor(this.area);

		// Change value LEFT, CENTER, RIGHT according to your requirement 
		this.position_in_panel = EyePosition.RIGHT;

		let children = null;
		switch (this.position_in_panel) {
			case EyePosition.LEFT:
			children = Main.panel._leftBox.get_children();
			Main.panel._leftBox.add(this.actor, children.length-1);
			break;
			case EyePosition.CENTER:
			Main.panel._centerBox.add(this.actor, { y_fill: true });
			break;
			case EyePosition.RIGHT:
			children = Main.panel._rightBox.get_children();
			Main.panel._rightBox.insert_child_at_index(this.actor, 0);
			break;
		}
	},

	setActive: function(enabled) {
		if (enabled) {
			this._repaint_handler = this.area.connect("repaint", Lang.bind(this, this._draw));
			this._update_handler = Mainloop.timeout_add(UPDATE_INTERVAL, Lang.bind(this, this._on_timeout));

			this.area.queue_repaint();
		} else {
		    if(this._repaint_handler)
				this.area.disconnect(this._repaint_handler);
		    if(this._update_handler)
				Mainloop.source_remove(this._update_handler);
		}
	}, 

	_on_timeout: function() {
		this.area.queue_repaint();
		return true;
	},

	_draw: function(area) {
		let [area_width, area_height] = area.get_surface_size();
		 
		let eye_rad = area_height / 2.3;
		let iris_rad = eye_rad*0.7;
		let pupil_rad = iris_rad*0.3;
		 
		let [area_x, area_y] = this._get_pos();
		area_x += area_width/2;
		area_y += area_height/2;
		 
		let [mouse_x, mouse_y, mask] = global.get_pointer();
		mouse_x -= area_x;
		mouse_y -= area_y;
		 
		let mouse_theta = -Math.atan2(mouse_x, mouse_y) +Math.PI/2;
		let mouse_rad = Math.sqrt(mouse_x*mouse_x + mouse_y*mouse_y);
		let max_rad = eye_rad*Math.cos( Math.asin((iris_rad+IRIS_WIDTH/2)/eye_rad) );
		if(mouse_rad > max_rad)
		mouse_rad = max_rad;
		 
		let cr = area.get_context();
		let theme_node = this.area.get_theme_node();
		Clutter.cairo_set_source_color(cr, theme_node.get_foreground_color());
		 
		cr.translate(area_width / 2.0, area_height / 2.0);
		 
		cr.setLineWidth(EYE_WIDTH);
		cr.arc(0, 0, eye_rad, 0, 2 * Math.PI);
		cr.stroke();       
		 
		cr.rotate(mouse_theta);
		cr.translate(mouse_rad,0);
		 
		cr.scale(Math.cos(Math.asin(mouse_rad/(eye_rad-EYE_WIDTH))),1);
		cr.setLineWidth(IRIS_WIDTH);
		cr.arc(0, 0, iris_rad - IRIS_WIDTH, 0, 2 * Math.PI);
		cr.stroke();
		 
		cr.setLineWidth(PUPIL_WIDTH);
		cr.arc(IRIS_WIDTH, 0, pupil_rad - PUPIL_WIDTH, 0, 2 * Math.PI);
		cr.fill();
		 
		cr.save();
		cr.restore();
	},

	_get_pos: function() {
		let area_x = 0;
		let area_y = 0;
		 
		let [tx, ty] = Main.panel.actor.get_position();
		area_x += tx;
		area_y += ty;
		 
		switch (this.position_in_panel) {
			case EyePosition.LEFT:
			[tx, ty] = Main.panel._leftBox.get_position();
			break;
			case EyePosition.CENTER:
			[tx, ty] = Main.panel._centerBox.get_position();
			break;
			case EyePosition.RIGHT:
			[tx, ty] = Main.panel._rightBox.get_position();
			break;
		}
		area_x += tx;
		area_y += ty;
		 
		[tx, ty] = this.actor.get_position();
		area_x += tx;
		area_y += ty;
		 
		[tx, ty] = this.area.get_position();
		area_x += tx;
		area_y += ty;
		 
		return [area_x, area_y];
	},
}

// vim: set ai ts=4 sw=4 tw=79:
